#!/bin/bash
#

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> [source]"
}


get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

product_dir=${1}
type=${2}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the product directory"
   usage
   exit 1
fi

upsver=v5_0_5
updver=v4_7_5
upsdotver=`echo ${upsver} | sed -e 's/_/./g' | sed -e 's/^v//'`

OS1=`uname`

if [ ${OS1} = "Darwin" ]
then
   plat=`uname -m`
   #OSnum=`uname -r | cut -f1 -d"."`
   #OS=${OS1}${OSnum}
    mactype=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep Mac | wc -w`
    if [ ${mactype} -gt 0 ]
    then
       macos=`system_profiler  -xml -detailLevel mini SPSoftwareDataType | grep Mac | cut -d ' ' -f 4 | cut -d . -f 1,2`
       macos1=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep Mac | cut -d ' ' -f 4 | cut -d . -f 1`
       macos2=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep Mac | cut -d ' ' -f 4 | cut -d . -f 2`
    else 
       macos=`system_profiler  -xml -detailLevel mini SPSoftwareDataType | grep "OS X" | cut -d ' ' -f 3 | cut -d . -f 1,2`
       macos1=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep "OS X" | cut -d ' ' -f 3 | cut -d . -f 1`
       macos2=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep "OS X" | cut -d ' ' -f 3 | cut -d . -f 2`
    fi
    OS=osx${macos1}${macos2}
else
   plat=`uname -p`
   OSnum=`lsb_release -r | cut -f2 | cut -f1 -d"."`
   OS=${OS1}${OSnum}
   if [ `lsb_release -d | cut  -f3 -d" "` = "SLF" ]
   then
      OS=slf${OSnum}
   fi
   if [ `lsb_release -i | cut -f2` = "ScientificFermi" ]
   then
      OS=slf${OSnum}
   fi
   if [ `lsb_release -i | cut -f2` = "CentOS" ]
   then
      OS=slf${OSnum}
   fi
   if [ `lsb_release -d | cut  -f4 -d" "` = "SLC" ]
   then
      OS=slc${OSnum}
   fi
   if [ `lsb_release -d | cut  -f4 -d" "` = "LTS" ]
   then
      OS=slf${OSnum}
   fi
fi

source ${product_dir}/setups
if [ ${OS1} = "Darwin" ]
then
   flvr=`ups flavor -2`
else
   flvr=`ups flavor`
fi

set -x

cd ${product_dir}

if [ "${type}" = "source" ]
then
   tar cjf ups-upd-${upsdotver}-source.tar.bz2 \
      .up* upd/${updver} \
      ups/${upsver}/*.sh ups/${upsver}/tar
else
   tar cjf ups-upd-${upsdotver}-${OS}-${plat}.tar.bz2 \
      setups setup setups_layout .up* \
      upd/${updver}*  upd/current.chain \
      ups/current.chain/${flvr}_ ups/${upsver}.version/${flvr}_ \
      ups/${upsver}/${flvr} 
fi

exit 0
